# Apply patches to source code
#
# Copyright (C) 2012, 2019 Patrick McDermott
#
# This file is part of opkbuild.
#
# opkbuild is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# opkbuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with opkbuild.  If not, see <http://www.gnu.org/licenses/>.

set -eu

apply_patches()
{
	local applied=
	local orig_lang=
	local patch=

	applied='false'

	if [ -d '../patches' ] && [ -d 'src' ]; then
		mkdir -p '.opkbuild'
		patches="${OB_LF}$(cat '.opkbuild/patches' 2>/dev/null || \
			:)${OB_LF}"
		exec 3>>'.opkbuild/patches'
		cd 'src/'
		# Iterate over patches ordered alphabetically by name.
		# POSIX.1-2008 says that the results of a pathname expansion
		# shall be "sorted according to the collating sequence in effect
		# in the current locale".  So for consistent results, we first
		# set a standard locale.
		orig_lang="${LANG}"
		LANG='POSIX'
		for patch in '../../patches/'*; do
			[ -f "${patch}" ] || continue
			LANG="${orig_lang}"
			patch="${patch#../../patches/}"
			case "${patches}" in *"${OB_LF}${patch}${OB_LF}"*)
				ob_info "$(ob_get_msg 'skipping_patch')" \
					"${patch}"
				continue
			esac
			ob_info "$(ob_get_msg 'applying_patch')" "${patch}"
			if ! patch -N -p 1 -u -i "../../patches/${patch}"; then
				ob_error "$(ob_get_msg 'cant_apply_patch')" \
					"${patch}"
				return 1
			fi
			printf '%s\n' "${patch}" >&3
			applied='true'
		done
		LANG="${orig_lang}"
		cd '../'
		exec 3>&-
	fi

	${applied} || ob_info "$(ob_get_msg 'no_patches')"

	return 0
}

main()
{
	if ! ob_set_text_domain 'opkbuild'; then
		printf '%s: Error: Failed to load locale messages\n' \
			"${0##*/}" >&2
		return 1
	fi

	ob_init_package '..' || return 1

	apply_patches || return 1

	return 0
}
